import { afterEach, beforeEach, describe, expect, it, vi } from 'vitest';
import shell from 'shelljs';
import PouchDB from 'pouchdb';
import { getRte, compactRteOrTrksegDebounced, putRte } from './rte';

import { putRtept } from './rtept';
import { get, getFamily, put } from './lib';
import { emptyWpt } from './wpt';

const test = it;
const jest = vi;

declare global {
  var db: any;
  var dbReady: boolean;
}

const originalDb = globalThis.db;
const originalDateNow = globalThis.Date.now;

describe('The rtept module with a real db', () => {
  beforeEach(async () => {
    globalThis.db = new PouchDB('dyomedea', {
      auto_compaction: false,
    });
    globalThis.Date.now = () => 0;
    vi.mock('compactRteOrTrksegDebounced', () => {
      console.log({ caller: 'mocked compactRteOrTrksegDebounced' });
    });
  });
  afterEach(async () => {
    try {
      await db.destroy();
    } catch (err) {
      //      console.error(err);
      await shell.exec('rm -rf  $*$');
    }
    globalThis.db = undefined;
    globalThis.dbReady = false;
    globalThis.Date.now = originalDateNow;
  });
  it('can write a rtept inside a rte', async () => {
    const idRte = 'whatever';
    const rtept: Wpt = {
      $: { lat: 2, lon: 2 },
    };
    const rte0: Rte = {
      name: 'A new route',
      rtept: [
        {
          $: { lat: 0, lon: 0 },
        },
        {
          $: { lat: 1, lon: 1 },
        },
      ],
    };
    const rte1: Rte = {
      name: 'A new route',
      rtept: [
        {
          $: { lat: 0, lon: 0 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
      ],
    };

    const rtePut = await putRte({ id: idRte, rte: rte0 });
    expect(rtePut).toEqual(rte0);
    const rtePutRtept = await putRtept({ id: idRte, index: 1, wpt: rtept });
    const rteGetRtept = await getRte({ id: idRte });
    expect(rteGetRtept).toEqual(rte1);
  });
  it('can write a rtept outside a rte', async () => {
    const idRte = 'gpx/4320836410265485/2rte/000034';
    const idRtept = 'gpx/4320836410265485/2rte/000034/000035';
    const rtept3: Wpt = {
      $: { lat: 3, lon: 3 },
    };
    const rtept4: Wpt = {
      $: { lat: 4, lon: 4 },
    };
    const rte0: Rte = {
      name: 'A new route',
      rtept: [
        {
          $: { lat: 1, lon: 1 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
      ],
    };
    const rte1: Rte = {
      name: 'A new route',
      rtept: [
        {
          $: { lat: 1, lon: 1 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
        rtept3,
      ],
    };
    const rte2: Rte = {
      name: 'A new route',
      rtept: [
        {
          $: { lat: 1, lon: 1 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
        rtept4,
      ],
    };

    const rtePut = await putRte({ id: idRte, rte: rte0 });
    expect(rtePut).toEqual(rte0);
    await put(idRtept, 'rtept', () => rtept3, emptyWpt);
    expect(await getRte({ id: idRte })).toEqual(rte1);
    await putRtept({
      id: idRte,
      index: 2,
      wpt: rtept4,
    });
    expect((await getFamily(idRte)).rows.length).toEqual(2);
    expect(await getRte({ id: idRte })).toEqual(rte2);
    expect((await get(idRtept)).doc).toEqual(rtept4);
  });
  it('can write a rtept outside a rte with a negative index', async () => {
    const idRte = 'gpx/4320836410265485/2rte/000034';
    const idRtept = 'gpx/4320836410265485/2rte/000034/000035';
    const rtept3: Wpt = {
      $: { lat: 3, lon: 3 },
    };
    const rtept4: Wpt = {
      $: { lat: 4, lon: 4 },
    };
    const rte0: Rte = {
      name: 'A new route',
      rtept: [
        {
          $: { lat: 1, lon: 1 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
      ],
    };
    const rte1: Rte = {
      name: 'A new route',
      rtept: [
        {
          $: { lat: 1, lon: 1 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
        rtept3,
      ],
    };
    const rte2: Rte = {
      name: 'A new route',
      rtept: [
        {
          $: { lat: 1, lon: 1 },
        },
        {
          $: { lat: 2, lon: 2 },
        },
        rtept4,
      ],
    };

    const rtePut = await putRte({ id: idRte, rte: rte0 });
    await put(idRtept, 'rtept', () => rtept3, emptyWpt);
    await putRtept({
      id: idRte,
      index: -1,
      wpt: rtept4,
    });
    expect(await getRte({ id: idRte })).toEqual(rte2);
    expect((await get(idRtept)).doc).toEqual(rtept4);
  });
});
